package de.tudresden.inf.st.ros2rag.receiverstub;

import com.beust.jcommander.JCommander;
import com.beust.jcommander.Parameter;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.InvalidProtocolBufferException;
import config.Dataconfig;
import config.Dataconfig.DataConfig;
import config.Robotconfig.RobotConfig;
import de.tudresden.inf.st.ros2rag.starter.ast.MqttUpdater;
import de.tudresden.inf.st.ros2rag.starter.data.DataConfiguration;
import de.tudresden.inf.st.ros2rag.starter.data.DataJoint;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import panda.Linkstate.PandaLinkState;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class Main {

  private static final Logger logger = LogManager.getLogger(Main.class);

  public static void main(String[] args) throws Exception {
    Main main = new Main();
    ObjectMapper mapper = new ObjectMapper();
    File configFile = new File(args[0]);
    System.out.println("Using config file: " + configFile.getAbsolutePath());
    DataConfiguration config = mapper.readValue(configFile, DataConfiguration.class);
    main.run(config);
  }

  private void run(DataConfiguration config) throws IOException, InterruptedException {
    final CountDownLatch finish = new CountDownLatch(1);

    MqttUpdater receiver = new MqttUpdater("receiver stub");
    receiver.setHost(config.mqttHost);
    receiver.waitUntilReady(2, TimeUnit.SECONDS);
    receiver.newConnection(config.robotConfigTopic, this::printRobotConfig);
    receiver.newConnection(config.dataConfigTopic, this::printDataConfig);
    for (DataJoint joint : config.joints) {
      receiver.newConnection(joint.topic, this::printPandaLinkState);
    }

    receiver.newConnection("components", bytes -> {
      String message = new String(bytes);
      logger.info("Components: {}", message);
    });

    receiver.newConnection("receiver", bytes -> {
      String message = new String(bytes);
      if (message.equals("exit")) {
        finish.countDown();
      }
    });

    Runtime.getRuntime().addShutdownHook(new Thread(receiver::close));
    if (config.exitAfterSeconds > 0) {
      finish.await(config.exitAfterSeconds, TimeUnit.SECONDS);
    } else {
      finish.await();
    }
    receiver.close();
  }

  private void printPandaLinkState(byte[] bytes) {
    try {
      logger.debug("Got a joint message, parsing ...");
      PandaLinkState pls = PandaLinkState.parseFrom(bytes);
      PandaLinkState.Position tmpPosition = pls.getPos();
      PandaLinkState.Orientation tmpOrientation = pls.getOrient();
      PandaLinkState.TwistLinear tmpTwistLinear = pls.getTl();
      PandaLinkState.TwistAngular tmpTwistAngular = pls.getTa();
      logger.info("{}: pos({},{},{}), orient({},{},{},{})," +
              " twist-linear({},{},{}), twist-angular({},{},{})",
          pls.getName(),
          tmpPosition.getPositionX(),
          tmpPosition.getPositionY(),
          tmpPosition.getPositionZ(),
          tmpOrientation.getOrientationX(),
          tmpOrientation.getOrientationY(),
          tmpOrientation.getOrientationZ(),
          tmpOrientation.getOrientationW(),
          tmpTwistLinear.getTwistLinearX(),
          tmpTwistLinear.getTwistLinearY(),
          tmpTwistLinear.getTwistLinearZ(),
          tmpTwistAngular.getTwistAngularX(),
          tmpTwistAngular.getTwistAngularY(),
          tmpTwistAngular.getTwistAngularZ());
    } catch (InvalidProtocolBufferException e) {
      e.printStackTrace();
    }
  }

  private void printRobotConfig(byte[] bytes) {
    try {
      logger.debug("Got a robotConfig message, parsing ...");
      RobotConfig robotConfig = RobotConfig.parseFrom(bytes);
      logger.info("robotConfig: speed = {}, loopTrajectory = {}, planningMode = {}",
          robotConfig.getSpeed(),
          robotConfig.getLoopTrajectory(),
          robotConfig.getPlanningMode().toString());
    } catch (InvalidProtocolBufferException e) {
      e.printStackTrace();
    }
  }

  private void printDataConfig(byte[] bytes) {
    try {
      logger.debug("Got a dataConfig message, parsing ...");
      DataConfig dataConfig = DataConfig.parseFrom(bytes);
      logger.info("dataConfig: position = {}, orientation = {}, twistLinear = {}, twistAngular = {}, publishRate = {}",
          dataConfig.getEnablePosition(),
          dataConfig.getEnableOrientation(),
          dataConfig.getEnableTwistLinear(),
          dataConfig.getEnableTwistAngular(),
          dataConfig.getPublishRate());
    } catch (InvalidProtocolBufferException e) {
      e.printStackTrace();
    }

  }

}
